@@ -22,8 +22,6 @@
 #import <Foundation/NSArray.h>
 #import <Foundation/NSCalendarDate.h>
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSLock.h>
 #import <Foundation/NSTimer.h>
 #import <Foundation/NSUserDefaults.h>
 #import <Foundation/NSValue.h>
@@ -33,7 +31,6 @@
 
 #import <GDLAccess/EOAdaptor.h>
 #import <GDLAccess/EOAdaptorContext.h>
-#import <GDLAccess/EOAdaptorChannel.h>
 
 #import "GCSChannelManager.h"
 #import "NSURL+GCS.h"
@@ -500,7 +497,7 @@ static NSTimeInterval ChannelCollectionTimer = 5 * 60;
   ms = [NSMutableString stringWithCapacity: 256];
   [ms appendFormat: @"<0x%p[%@]: ", self, NSStringFromClass ([self class])];
 
-  [ms appendFormat: @" #adaptors=%d", [urlToAdaptor count]];
+  [ms appendFormat: @" #adaptors=%d", (int)[urlToAdaptor count]];
 
   [ms appendString: @">"];
   return ms;
