@@ -557,7 +557,7 @@
         {
 	  iCalPerson *person;
 	  NSArray *members;
-	  NSDictionary *user;
+	  SOGoUser *user;
           id <SOGoSource> source;
 	  
 	  // We did decompose a group...
@@ -574,10 +574,10 @@
 
                   // If the organizer is part of the group, we skip it from
                   // the addition to the attendees' list
-                  if ([[user objectForKey: @"c_emails"] containsObject: organizerEmail])
+                  if ([user hasEmail: organizerEmail])
                     continue;
 
-                  person = [self iCalPersonWithUID: [user objectForKey: @"c_uid"]];
+                  person = [self iCalPersonWithUID: [user login]];
                   [person setTag: @"ATTENDEE"];
                   [person setParticipationStatus: [currentAttendee participationStatus]];
                   [person setRsvp: [currentAttendee rsvp]];
@@ -587,6 +587,12 @@
                     [allAttendees addObject: person];
                 }
             }
+          else
+            {
+              [self errorWithFormat: @"Inconsistency error - got group identifier (%@) from a source (%@) that does not support groups.", [currentAttendee rfc822Email], [dict objectForKey: @"SOGoSource"]];
+              RELEASE(pool);
+              return NO;
+            }
 	}
       
       j++;
@@ -822,7 +828,7 @@
 
           // No organizer, grab the event's owner
           if (![senderEmail length])
-            senderEmail = shortSenderEmail = [[ownerUser defaultIdentity] objectForKey: @"email"];
+            senderEmail = shortSenderEmail = [[ownerUser primaryIdentity] objectForKey: @"email"];
 
           /* calendar part */
           eventBodyPart = [self _bodyPartForICalObject: object];
@@ -975,7 +981,8 @@
   [headerMap setObject: @"1.0" forKey: @"MIME-Version"];
 
   if (textOnly)
-    [headerMap setObject: @"text/html" forKey: @"content-type"];
+    [headerMap setObject: @"text/html; charset=utf-8"
+                  forKey: @"content-type"];
   else
     [headerMap setObject: @"multipart/mixed" forKey: @"content-type"];
 
