@@ -1,6 +1,6 @@
 /* iCalEntityObject+SOGo.h - this file is part of SOGo
  *
- * Copyright (C) 2007-2014 Inverse inc.
+ * Copyright (C) 2007-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 @class iCalAlarm;
 @class NSMutableDictionary;
 @class SOGoUser;
+@class WOContext;
 
 extern NSCalendarDate *iCalDistantFuture;
 extern NSNumber *iCalDistantFutureNumber;
@@ -34,12 +35,18 @@ extern NSNumber *iCalDistantFutureNumber;
 
 + (void) initializeSOGoExtensions;
 
+- (NSDictionary *) attributesInContext: (WOContext *) context;
+- (void) setAttributes: (NSDictionary *) data
+             inContext: (WOContext *) context;
+
 - (BOOL) userIsAttendee: (SOGoUser *) user;
 - (BOOL) userIsOrganizer: (SOGoUser *) user;
 
 - (iCalPerson *) userAsAttendee: (SOGoUser *) user;
 
-- (NSArray *) attendeeUIDs;
+- (iCalPerson *) participantForUser:  (SOGoUser *) theUser
+                           attendee: (iCalPerson *) theAttendee;
+/* - (NSArray *) attendeeUIDs; */
 - (BOOL) isStillRelevant;
 
 - (id) itipEntryWithMethod: (NSString *) method;
@@ -47,20 +54,26 @@ extern NSNumber *iCalDistantFutureNumber;
 - (NSArray *) attendeesWithoutUser: (SOGoUser *) user;
 
 - (int) priorityNumber;
-- (NSString *) createdBy;
+- (NSDictionary *) createdBy;
+
+- (void) adjustDate: (NSCalendarDate **) date
+     withTimeString: (NSString *) timeString
+          inContext: (WOContext *) context;
 
 - (NSNumber *) quickRecordDateAsNumber: (NSCalendarDate *) _date
 			    withOffset: (int) offset
 			     forAllDay: (BOOL) allDay;
 
 - (NSMutableDictionary *) quickRecordFromContent: (NSString *) theContent
-                                       container: (id) theContainer;
+                                       container: (id) theContainer
+				 nameInContainer: (NSString *) nameInContainer;
 
 - (iCalAlarm *) firstSupportedAlarm;
 - (iCalAlarm *) firstDisplayOrAudioAlarm;
 
 - (void) updateNextAlarmDateInRow: (NSMutableDictionary *) row
-                     forContainer: (id) theContainer;
+                     forContainer: (id) theContainer
+		  nameInContainer: (NSString *) nameInContainer;
 
 @end
 
