@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2009-2019 Inverse inc.
+  Copyright (C) 2009-2020 Inverse inc.
 
   This file is part of SOGo.
 
@@ -28,7 +28,7 @@
   SOGoMailAccount
     Parent object: SOGoMailAccounts
     Child objects: SOGoMailFolder
-  
+
   The SOGoMailAccount represents a single IMAP4 mail account (host, login,
   password, etc)
 */
@@ -87,8 +87,11 @@ typedef enum {
                             forceActivation: (BOOL) forceActivation;
 
 - (NSArray *) identities;
+- (NSDictionary *) defaultIdentity;
+- (NSDictionary *) identityForEmail: (NSString *) email;
 - (NSString *) signature;
 - (NSString *) encryption;
+- (NSString *) tlsVerifyMode;
 
 /* folder pathes */
 - (NSArray *) toManyRelationshipKeysWithNamespaces: (BOOL) withNSs;
