@@ -109,6 +109,8 @@ static Class NSStringK;
 
       _passwordPolicy = NO;
       _updateSambaNTLMPasswords = NO;
+      _lookupFields = [NSArray arrayWithObject: @"*"];
+      [_lookupFields retain];
 
       _kindField = nil;
       _multipleBookingsField = nil;
@@ -157,6 +159,7 @@ static Class NSStringK;
   [_MSExchangeHostname release];
   [_modifiers release];
   [_displayName release];
+  [_lookupFields release];
   [super dealloc];
 }
 
@@ -192,6 +195,7 @@ static Class NSStringK;
        IMAPLoginField: [udSource objectForKey: @"IMAPLoginFieldName"]
        SieveHostField: [udSource objectForKey: @"SieveHostFieldName"]
            bindFields: [udSource objectForKey: @"bindFields"]
+         lookupFields: [udSource objectForKey: @"lookupFields"]
             kindField: [udSource objectForKey: @"KindFieldName"]
             andMultipleBookingsField: [udSource objectForKey: @"MultipleBookingsFieldName"]];
 
@@ -227,7 +231,7 @@ static Class NSStringK;
       if (!_contactInfoAttribute)
         _contactInfoAttribute = [dd ldapContactInfoAttribute];
       [_contactInfoAttribute retain];
-      
+
       udQueryLimit = [udSource objectForKey: @"SOGoLDAPQueryLimit"];
       if (udQueryLimit)
         _queryLimit = [udQueryLimit intValue];
@@ -263,7 +267,7 @@ static Class NSStringK;
 
       if ([udSource objectForKey: @"updateSambaNTLMPasswords"])
         _updateSambaNTLMPasswords = [[udSource objectForKey: @"updateSambaNTLMPasswords"] boolValue];
-      
+
       ASSIGN(_MSExchangeHostname, [udSource objectForKey: @"MSExchangeHostname"]);
     }
 
@@ -331,6 +335,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
     IMAPLoginField: (NSString *) newIMAPLoginField
     SieveHostField: (NSString *) newSieveHostField
         bindFields: (id) newBindFields
+      lookupFields: (NSArray *) newLookupFields
          kindField: (NSString *) newKindField
   andMultipleBookingsField: (NSString *) newMultipleBookingsField
 {
@@ -377,6 +382,8 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
           ASSIGN(_bindFields, [newBindFields componentsSeparatedByString: @","]);
         }
     }
+  if (newLookupFields)
+    ASSIGN(_lookupFields, newLookupFields);
   if (newKindField)
     ASSIGN(_kindField, [newKindField lowercaseString]);
   if (newMultipleBookingsField)
@@ -585,7 +592,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
                   didBind = [bindConnection bindWithMethod: @"simple"
                                                     binddn: userDN
                                                credentials: _pwd];
-                else  
+                else
                   didBind = [bindConnection bindWithMethod: @"simple"
                                                     binddn: userDN
                                                credentials: _pwd
@@ -642,14 +649,14 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
   NSArray *changes;
 
   BOOL didChange;
-  
+
   attr = [[NGLdapAttribute alloc] initWithAttributeName: theAttribute];
   [attr addStringValue: theValue];
-  
+
   mod = [NGLdapModification replaceModification: attr];
-  
+
   changes = [NSArray arrayWithObject: mod];
-  
+
   if ([bindConnection bindWithMethod: @"simple"
                               binddn: theUserDN
                          credentials: theUserPassword])
@@ -659,7 +666,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
     }
   else
     didChange = NO;
-  
+
   RELEASE(attr);
 
   return didChange;
@@ -672,7 +679,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
                     oldPassword: (NSString *) oldPassword
                     newPassword: (NSString *) newPassword
                            perr: (SOGoPasswordPolicyError *) perr
-  
+
 {
   NGLdapConnection *bindConnection;
   NSString *userDN;
@@ -721,7 +728,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
                     // We don't use a password policy - we simply use
                     // a modify-op to change the password
                     NSString* encryptedPass;
-                    
+
                     if ([_userPasswordAlgorithm isEqualToString: @"none"])
                       {
                         encryptedPass = newPassword;
@@ -730,7 +737,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
                       {
                         encryptedPass = [self _encryptPassword: newPassword];
                       }
-                    
+
                     if (encryptedPass != nil)
                       {
                         *perr = PolicyNoError;
@@ -750,7 +757,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
                                         userDN: userDN
                                       password: newPassword
                                     connection: bindConnection];
-                    
+
                     [self _ldapModifyAttribute: @"sambaLMPassword"
                                      withValue: [newPassword asLMHash]
                                         userDN: userDN
@@ -773,7 +780,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
         }
     }
   NS_ENDHANDLER ;
-  
+
   [bindConnection release];
   return didChange;
 }
@@ -917,11 +924,11 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
     [qs appendFormat: @" AND %@", _filter];
   qualifier = [EOQualifier qualifierWithQualifierFormat: qs];
 
-  if ([_scope caseInsensitiveCompare: @"BASE"] == NSOrderedSame) 
+  if ([_scope caseInsensitiveCompare: @"BASE"] == NSOrderedSame)
     entries = [ldapConnection baseSearchAtBaseDN: _baseDN
                                        qualifier: qualifier
                                       attributes: attributes];
-  else if ([_scope caseInsensitiveCompare: @"ONE"] == NSOrderedSame) 
+  else if ([_scope caseInsensitiveCompare: @"ONE"] == NSOrderedSame)
     entries = [ldapConnection flatSearchAtBaseDN: _baseDN
                                        qualifier: qualifier
                                       attributes: attributes];
@@ -1125,7 +1132,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
   ldifRecord = [ldapEntry asDictionary];
   [ldifRecord setObject: self forKey: @"source"];
   [ldifRecord setObject: [ldapEntry dn] forKey: @"dn"];
-  
+
   // We get our objectClass attribute values. We lowercase
   // everything for ease of search after.
   o = [ldapEntry objectClasses];
@@ -1253,7 +1260,6 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
   NSEnumerator *entries;
   NSMutableArray *contacts;
   EOQualifier *qualifier;
-  NSArray *attributes;
 
   contacts = [NSMutableArray array];
 
@@ -1261,20 +1267,19 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
     {
       ldapConnection = [self _ldapConnection];
       qualifier = [self _qualifierForFilter: match onCriteria: criteria];
-      attributes = [NSArray arrayWithObject: @"*"];
 
       if ([_scope caseInsensitiveCompare: @"BASE"] == NSOrderedSame)
         entries = [ldapConnection baseSearchAtBaseDN: _baseDN
                                            qualifier: qualifier
-                                          attributes: attributes];
+                                          attributes: _lookupFields];
       else if ([_scope caseInsensitiveCompare: @"ONE"] == NSOrderedSame)
         entries = [ldapConnection flatSearchAtBaseDN: _baseDN
                                            qualifier: qualifier
-                                          attributes: attributes];
-      else /* we do it like before */ 
+                                          attributes: _lookupFields];
+      else /* we do it like before */
         entries = [ldapConnection deepSearchAtBaseDN: _baseDN
                                            qualifier: qualifier
-                                          attributes: attributes];
+                                          attributes: _lookupFields];
       while ((currentEntry = [entries nextObject]))
         [contacts addObject:
                     [self _convertLDAPEntryToContact: currentEntry]];
@@ -1287,23 +1292,21 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
 {
   NGLdapConnection *ldapConnection;
   NSEnumerator *entries;
-  NSArray *attributes;
 
   ldapConnection = [self _ldapConnection];
-  attributes = [NSArray arrayWithObject: @"*"];
 
   if ([_scope caseInsensitiveCompare: @"BASE"] == NSOrderedSame)
     entries = [ldapConnection baseSearchAtBaseDN: _baseDN
                                        qualifier: theQualifier
-                                      attributes: attributes];
+                                      attributes: _lookupFields];
   else if ([_scope caseInsensitiveCompare: @"ONE"] == NSOrderedSame)
     entries = [ldapConnection flatSearchAtBaseDN: _baseDN
                                        qualifier: theQualifier
-                                      attributes: attributes];
+                                      attributes: _lookupFields];
   else
     entries = [ldapConnection deepSearchAtBaseDN: _baseDN
                                        qualifier: theQualifier
-                                      attributes: attributes];
+                                      attributes: _lookupFields];
 
   return [entries nextObject];
 }
@@ -1357,7 +1360,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
   NGLdapEntry *entry;
   EOQualifier *qualifier;
   NSString *login;
-  
+
   login = nil;
   qualifier = nil;
 
@@ -1410,24 +1413,24 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
   EOQualifier *qualifier;
   NGLdapEntry *ldapEntry;
   NSString *s;
-    
+
   if ([theValue length] > 0 && [theAttributes count] > 0)
     {
       if ([theAttributes count] == 1)
 	{
 	    s = [NSString stringWithFormat: @"(%@='%@')",
 			  [theAttributes lastObject], SafeLDAPCriteria(theValue)];
-	    
+
 	}
       else
 	{
 	  NSString *fieldFormat;
-	  
+
 	  fieldFormat = [NSString stringWithFormat: @"(%%@='%@')", SafeLDAPCriteria(theValue)];
 	  s = [[theAttributes stringsWithFormat: fieldFormat]
 			 componentsJoinedByString: @" OR "];
 	}
-      
+
       qualifier = [EOQualifier qualifierWithQualifierFormat: s];
       ldapEntry = [self _lookupLDAPEntry: qualifier];
     }
@@ -1824,6 +1827,7 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
 		  IMAPLoginField: nil
 		  SieveHostField: nil
 		 bindFields: nil
+		lookupFields: nil
 		  kindField: nil
 		  andMultipleBookingsField: nil];
               [ab setListRequiresDot: NO];
@@ -1954,7 +1958,7 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
                                      reason: @"user addressbooks"
                           @" are not supported"
                                    userInfo: nil];
-  
+
   return result;
 }
 
@@ -2027,9 +2031,9 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
   NSMutableArray *dns, *uids, *logins;
   NSString *dn, *login;
   SOGoUserManager *um;
-  NSDictionary *d;
+  NSDictionary *d, *contactInfos;
   SOGoUser *user;
-  NSArray *o;
+  NSArray *o, *users;
   NSAutoreleasePool *pool;
   int i, c;
   NGLdapEntry *entry;
@@ -2081,8 +2085,7 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
               if (user)
                 {
                   [logins addObject: login];
-                  [members addObject: [NSDictionary dictionaryWithObject: login
-                                                                  forKey: @"c_uid"]];
+                  [members addObject: user];
                 }
               [pool release];
             }
@@ -2095,16 +2098,24 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
               user = [SOGoUser userWithLogin: login  roles: nil];
               if (user)
                 {
-                  [logins addObject: login];
-                  [members addObject: [NSDictionary dictionaryWithObject: login
-                                                                  forKey: @"c_uid"]];
+                  contactInfos = [self lookupContactEntryWithUIDorEmail: login inDomain: nil];
+                  if ([contactInfos objectForKey: @"isGroup"])
+                    {
+                      users = [self membersForGroupWithUID: login];
+                      [members addObjectsFromArray: users];
+                    }
+                  else
+                    {
+                      [logins addObject: login];
+                      [members addObject: user];
+                    }
                 }
               [pool release];
             }
 
 
           // We are done fetching members, let's cache the members of the group
-          // (ie., their UIDs) in memcached to speed up -hasMemberWithUID.
+          // (ie., their UIDs) in memcached to speed up -groupWithUIDHasMemberWithUID.
           [[SOGoCache sharedCache] setValue: [logins componentsJoinedByString: @","]
             forKey: [NSString stringWithFormat: @"%@+%@", uid, _domain]];
         }
