@@ -35,6 +35,10 @@
                              withSalt: (NSData *) theSalt
                               keyPath: (NSString *) theKeyPath;
 
+- (BOOL) verifyUsingScheme: (NSString *) passwordScheme
+                withPassword: (NSData *) thePassword
+                     keyPath: (NSString *) theKeyPath;
+
 - (NSData *) asLM;
 - (NSData *) asMD4;
 - (NSData *) asMD5;
@@ -50,14 +54,19 @@
 - (NSData *) asSymAES128CBCUsingIV: (NSString *) theIV
                            keyPath: (NSString *) theKeyPath;
 - (NSData *) asCramMD5;
-
+- (NSData *) asPBKDF2SHA1UsingSalt: (NSData *) theSalt;
 - (NSData *) asCryptUsingSalt: (NSData *) theSalt;
 - (NSData *) asMD5CryptUsingSalt: (NSData *) theSalt;
+- (NSData *) asBlowfishCryptUsingSalt: (NSData *) theSalt;
+#ifdef HAVE_SODIUM
+- (NSData *) asArgon2iUsingSalt: (NSData *) theSalt;
+- (NSData *) asArgon2idUsingSalt: (NSData *) theSalt;
+#endif /* HAVE_SODIUM */
 
 - (NSData *) extractSalt: (NSString *) theScheme;
 
 + (NSData *) generateSaltForLength: (unsigned int) theLength
-                        withBase64: (BOOL) doBase64;
+                        withPrintable: (BOOL) doPrintable;
 + (NSData *) generateSaltForLength: (unsigned int) theLength;
 
 + (NSString *) encodeDataAsHexString: (NSData *) theData;
