@@ -926,6 +926,7 @@ static NSArray *childRecordFields = nil;
   NSString *subscriptionPointer, *domain;
   NSMutableArray *allUsers;
   SOGoUserSettings *us;
+  SOGoUser *sogoUser;
   NSDictionary *dict;
   BOOL rc;
   int i;
@@ -941,22 +942,26 @@ static NSArray *childRecordFields = nil;
       source = [[SOGoUserManager sharedUserManager] sourceWithID: [dict objectForKey: @"SOGoSource"]];
       if ([source conformsToProtocol:@protocol(SOGoMembershipSource)])
         {
-          NSArray *members = [(id<SOGoMembershipSource>)(source) membersForGroupWithUID: [dict objectForKey: @"c_uid"]];
-          allUsers = [NSMutableArray array];
+          NSArray *members;
+
+          members = [(id<SOGoMembershipSource>)(source) membersForGroupWithUID: [dict objectForKey: @"c_uid"]];
+          allUsers = [NSMutableArray arrayWithArray: members];
 
-          for (i = 0; i < [members count]; i++)
-            {
-              [allUsers addObject: [[members objectAtIndex: i] objectForKey: @"c_uid"]];
-            }
           // We remove the active user from the group (if present) in order to
           // not subscribe him to their own resource!
-          [allUsers removeObject: [[context activeUser] login]];
+          [allUsers removeObject: [context activeUser]];
+        }
+      else
+        {
+          [self errorWithFormat: @"Inconsistency (subscribeUserOrGroup:reallyDo:response:) error - got group identifier (%@) from a source (%@) that does not support groups (%@).", theIdentifier, [dict objectForKey: @"SOGoSource"], NSStringFromClass([source class])];
+          return NO;
         }
     }
   else
     {
-      if (dict)
-	allUsers = [NSArray arrayWithObject: [dict objectForKey: @"c_uid"]];
+      sogoUser = [SOGoUser userWithLogin: theIdentifier roles: nil];
+      if (sogoUser)
+        allUsers = [NSArray arrayWithObject: sogoUser];
       else
 	allUsers = [NSArray array];
     }
@@ -969,7 +974,8 @@ static NSArray *childRecordFields = nil;
 
   for (i = 0; i < [allUsers count]; i++)
     {
-      us = [SOGoUserSettings settingsForUser: [allUsers objectAtIndex: i]];
+      sogoUser = [allUsers objectAtIndex: i];
+      us = [sogoUser userSettings];
       moduleSettings = [us objectForKey: [container nameInContainer]];
       if (!(moduleSettings
             && [moduleSettings isKindOfClass: [NSMutableDictionary class]]))
@@ -1115,6 +1121,7 @@ static NSArray *childRecordFields = nil;
     {
       davSQLFieldsTable = [NSMutableDictionary new];
       [davSQLFieldsTable setObject: @"c_version" forKey: @"{DAV:}getetag"];
+      [davSQLFieldsTable setObject: @"c_version" forKey: @"{http://calendarserver.org/ns/}getetag"];
       [davSQLFieldsTable setObject: @"" forKey: @"{DAV:}getcontenttype"];
       [davSQLFieldsTable setObject: @"" forKey: @"{DAV:}resourcetype"];
     }
@@ -1626,7 +1633,7 @@ static NSArray *childRecordFields = nil;
                       matchingUID: (NSString *) uid
 {
   int count, max;
-  NSDictionary *record;
+  NSDictionary *record, *dict;
   NSString *currentUID, *domain;
   NSMutableArray *acls;
 
@@ -1640,14 +1647,25 @@ static NSArray *childRecordFields = nil;
       if ([currentUID hasPrefix: @"@"])
         {
           domain = [[context activeUser] domain];
-          NSString *dict = [[SOGoUserManager sharedUserManager] contactInfosForUserWithUIDorEmail: currentUID
-                                                                                         inDomain: domain];
+          dict = [[SOGoUserManager sharedUserManager] contactInfosForUserWithUIDorEmail: currentUID
+                                                                               inDomain: domain];
           if (dict)
             {
-              id <SOGoSource> source = [[SOGoUserManager sharedUserManager] sourceWithID: [dict objectForKey: @"SOGoSource"]];
-              if ([source conformsToProtocol:@protocol(SOGoMembershipSource)] &&
-                  [(id<SOGoMembershipSource>)(source) groupWithUIDHasMemberWithUID: currentUID memberUid: uid])
-                [acls addObject: [record valueForKey: @"c_role"]];
+              id <SOGoSource> source;
+
+              source = [[SOGoUserManager sharedUserManager] sourceWithID: [dict objectForKey: @"SOGoSource"]];
+              if ([source conformsToProtocol:@protocol(SOGoMembershipSource)])
+                {
+                  if ([(id<SOGoMembershipSource>)(source) groupWithUIDHasMemberWithUID: currentUID memberUid: uid])
+                    [acls addObject: [record valueForKey: @"c_role"]];
+                  else
+                    [self errorWithFormat: @"Group %@ has no member with UID %@", currentUID, uid];
+                }
+              else
+                {
+                  [self errorWithFormat: @"Inconsistency (_aclsFromGroupRoles:matchingUID:) error - got group identifier (%@) from a source (%@) that does not support groups (%@).", currentUID, [dict objectForKey: @"SOGoSource"], NSStringFromClass([source class])];
+                  return [NSArray array];
+                }
             }
         }
     }
@@ -1780,7 +1798,7 @@ static NSArray *childRecordFields = nil;
               if ([source conformsToProtocol:@protocol(SOGoMembershipSource)])
                 {
                   NSArray *members;
-                  NSDictionary *user;
+                  SOGoUser *user;
                   unsigned int j;
 
                   // Fetch members to remove them from the cache along the group
@@ -1788,7 +1806,7 @@ static NSArray *childRecordFields = nil;
                   for (j = 0; j < [members count]; j++)
                     {
                       user = [members objectAtIndex: j];
-                      [groupsMembers addObject: [user objectForKey: @"c_uid"]];
+                      [groupsMembers addObject: [user login]];
                     }
 
                   if (![uid hasPrefix: @"@"])
@@ -1796,6 +1814,11 @@ static NSArray *childRecordFields = nil;
                     [usersAndGroups replaceObjectAtIndex: i
                                               withObject: [NSString stringWithFormat: @"@%@", uid]];
                 }
+              else
+                {
+                  [self errorWithFormat: @"Inconsistency error (removeAclsForUsers:forObjectAtPath:) - got group identifier (%@) from a source (%@) that does not support groups (%@).", uid, [dict objectForKey: @"SOGoSource"], NSStringFromClass([source class])];
+                  return;
+                }
             }
         }
       objectPath = [objectPathArray componentsJoinedByString: @"/"];
