@@ -39,6 +39,7 @@
                inDomain: (NSString *) domain;
 
 - (NSString *) domain;
+- (NSArray *) searchFields;
 
 /* requires a "." to obtain the full list of contacts */
 - (void) setListRequiresDot: (BOOL) aBool;
@@ -63,6 +64,7 @@
 - (NSArray *) allEntryIDs;
 - (NSArray *) allEntryIDsVisibleFromDomain: (NSString *) domain;
 - (NSArray *) fetchContactsMatching: (NSString *) filter
+                       withCriteria: (NSArray *) criteria
                            inDomain: (NSString *) domain;
 
 - (void) setSourceID: (NSString *) newSourceID;
@@ -100,6 +102,7 @@
 
 - (void) setBindDN: (NSString *) theDN;
 - (NSString *) bindDN;
+
 - (void) setBindPassword: (NSString *) thePassword;
 - (NSString *) bindPassword;
 - (BOOL) bindAsCurrentUser;
@@ -111,6 +114,14 @@
 - (NSString *) MSExchangeHostname;
 
 - (NSArray *) groupObjectClasses;
+
+- (void) updateBaseDNFromLogin: (NSString *) theLogin;
+@end
+
+@protocol SOGoMembershipSource <SOGoSource>
+- (NSArray *) membersForGroupWithUID: (NSString *) uid;
+- (BOOL) groupWithUIDHasMemberWithUID: (NSString *) uid
+                            memberUid: (NSString *) memberUid;
 @end
 
 #endif /* SOGOSOURCE_H */
