@@ -107,11 +107,10 @@ class WebDAVClient:
         self.conn.request(query.method, query.url,
                           body, self.prepare_headers(query, body))
         try:
-            query.set_response(self.conn.getresponse());
+            query.set_response(self.conn.getresponse())
         except httplib.BadStatusLine, e:
             print e
-            time.sleep(3)
-            query.set_response(self.conn.getresponse());
+            query.set_response(self.conn.getresponse())
         query.duration = time.time() - query.start
 
 class HTTPSimpleQuery:
