@@ -320,9 +320,9 @@
 
   o = [card nickname];
   if (o) [data setObject: o forKey: @"nickname"];
-  o = [card title];
-  if ([o length] > 0)
-    [data setObject: o forKey: @"title"];
+  o = [card titles];
+  if ([o count])
+    [data setObject: [o componentsJoinedByString: @" / "] forKey: @"title"];
   o = [card role];
   if ([o length] > 0)
     [data setObject: o forKey: @"role"];
@@ -397,7 +397,7 @@
   NSString *email;
   SOGoObject <SOGoContactObject> *contact;
   SOGoObject <SOGoSource> *source;
-  NSDictionary *user;
+  SOGoUser *user;
   id <WOActionResults> result;
   unsigned int i, max;
 
@@ -418,14 +418,14 @@
             {
               user = [allUsers objectAtIndex: i];
               allUserEmails = [NSMutableArray array];
-              emails = [[user objectForKey: @"c_emails"] objectEnumerator];
+              emails = [[user allEmails] objectEnumerator];
               while ((email = [emails nextObject])) {
                 [allUserEmails addObject: [NSDictionary dictionaryWithObjectsAndKeys:
                                                           email, @"value", @"work", @"type", nil]];
               }
               userData = [NSDictionary dictionaryWithObjectsAndKeys:
-                                       [user objectForKey: @"c_uid"], @"c_uid",
-                                       [user objectForKey: @"c_cn"], @"c_cn",
+                                       [user loginInDomain], @"c_uid",
+                                       [user cn], @"c_cn",
                                        allUserEmails, @"emails", nil];
               [allUsersData addObject: userData];
             }
