@@ -27,20 +27,11 @@
 
 @interface UIxContactsUserRightsEditor : UIxUserRightsEditor
 
-- (void) setUserCanCreateObjects: (BOOL) userCanCreateObjects;
 - (BOOL) userCanCreateObjects;
-
-- (void) setUserCanEraseObjects: (BOOL) userCanEraseObjects;
 - (BOOL) userCanEraseObjects;
-
-- (void) setUserCanEditObjects: (BOOL) userCanEditObjects;
 - (BOOL) userCanEditObjects;
-
-- (void) setUserCanViewObjects: (BOOL) userCanViewObjects;
 - (BOOL) userCanViewObjects;
 
-- (void) updateRights;
-
 @end
 
 #endif /* UIXCONTACTSUSERRIGHTSEDITOR_H */
