@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2007-2015 Inverse inc.
+  Copyright (C) 2007-2018 Inverse inc.
   Copyright (C) 2004 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -23,7 +23,13 @@
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSNull.h>
 
+#import <SOGo/NSObject+Utilities.h>
+
 #import <NGExtensions/NSObject+Logs.h>
+#import <NGMail/NGMimeMessageParser.h>
+#import <NGMime/NGMimeBodyPart.h>
+#import <NGMime/NGMimeMultipartBody.h>
+#import <NGMime/NGMimeType.h>
 
 #import <UI/MailerUI/WOContext+UIxMailer.h>
 
@@ -35,7 +41,7 @@
 
   Display multipart/alternative parts. Most common application is for messages
   which contain text/html and text/plain, but it is also used in other
-  contexts, eg in OGo appointment mails.
+  contexts, eg in SOGo appointment mails.
 
   TODO: We might want to give the user the possibility to access all parts
         of the alternative set.
@@ -57,22 +63,35 @@
 - (NSArray *) childPartTypes
 {
   NSMutableArray *types;
-  NSUInteger i, count;
   NSArray  *childParts;
+  NSString *mt, *st;
 
-  childParts = [[self bodyInfo] valueForKey:@"parts"];
-  count      = [childParts count];
-  types      = [NSMutableArray arrayWithCapacity:count];
+  NSUInteger i, count;
 
-  for (i = 0; i < count; i++) {
-    NSString *mt, *st;
+  if ([self decodedFlatContent])
+    childParts = [[self decodedFlatContent] parts];
+  else
+    childParts = [[self bodyInfo] valueForKey: @"parts"];
+
+  count = [childParts count];
+  types = [NSMutableArray arrayWithCapacity: count];
+
+  for (i = 0; i < count; i++)
+    {
+      if ([self decodedFlatContent])
+        {
+          mt = [[[childParts objectAtIndex: i] contentType] type];
+          st = [[[childParts objectAtIndex: i] contentType] subType];
+        }
+      else
+        {
+          mt = [[[childParts objectAtIndex: i] valueForKey: @"type"] lowercaseString];
+          st = [[[childParts objectAtIndex: i] valueForKey: @"subtype"] lowercaseString];
+        }
+      mt = [[mt stringByAppendingString: @"/"] stringByAppendingString: st];
+      [types addObject: (mt ? (id)mt : (id)[NSNull null])];
+    }
 
-    mt = [[[childParts objectAtIndex:i] valueForKey:@"type"] lowercaseString];
-    st = [[[childParts objectAtIndex:i] valueForKey:@"subtype"]
-	               lowercaseString];
-    mt = [[mt stringByAppendingString:@"/"] stringByAppendingString:st];
-    [types addObject:mt ? (id)mt : (id)[NSNull null]];
-  }
   return types;
 }
 
@@ -125,19 +144,24 @@
 {
   NSUInteger idx;
 
-  [childInfo release]; childInfo = nil;
+  DESTROY(childInfo);
   childIndex = 0;
 
   idx = [self _selectPartIndexFromTypes: [self childPartTypes]];
   if (idx == NSNotFound)
     {
-      [self errorWithFormat:@"could not select a part of types: %@",
-            [self childPartTypes]];
+      [self errorWithFormat:@"could not select a part of types: %@", [self childPartTypes]];
       return;
     }
 
   childIndex = idx + 1;
-  childInfo = [[[[self bodyInfo] valueForKey:@"parts"] objectAtIndex:idx] retain];
+
+  if ([self decodedFlatContent])
+    childInfo = [[[[self decodedFlatContent] parts] objectAtIndex: idx] bodyInfo];
+  else
+    childInfo = [[[self bodyInfo] valueForKey:@"parts"] objectAtIndex: idx];
+
+  [childInfo retain];
 }
 
 /* nested viewers */
@@ -147,28 +171,40 @@
   id info;
 
   info = [self childInfo];
-  return [[[self context] mailRenderingContext] viewerForBodyInfo:info];
+  return [[[self context] mailRenderingContext] viewerForBodyInfo: info];
 }
 
-- (id) renderedPart {
+- (id) renderedPart
+{
   id info, viewer;
   NSArray *parts;
   NSMutableArray *renderedParts;
   NSString *preferredType;
   NSUInteger i, max;
 
-  parts = [[self bodyInfo] objectForKey: @"parts"];
+  if ([self decodedFlatContent])
+    parts = [[self decodedFlatContent] parts];
+  else
+    parts = [[self bodyInfo] valueForKey: @"parts"];
+
   max = [parts count];
   renderedParts = [NSMutableArray arrayWithCapacity: max];
   for (i = 0; i < max; i++)
     {
       [self setChildIndex: i];
-      [self setChildInfo: [parts objectAtIndex: i]];
+
+      if ([self decodedFlatContent])
+        [self setChildInfo: [[parts objectAtIndex: i] bodyInfo]];
+      else
+        [self setChildInfo: [parts objectAtIndex: i]];
+
       info = [self childInfo];
       viewer = [[[self context] mailRenderingContext] viewerForBodyInfo: info];
       [viewer setBodyInfo: info];
       [viewer setPartPath: [self childPartPath]];
       [viewer setAttachmentIds: attachmentIds];
+      if ([self decodedFlatContent])
+        [viewer setDecodedContent: [parts objectAtIndex: i]];
       [renderedParts addObject: [viewer renderedPart]];
     }
 
