@@ -1,6 +1,6 @@
 /* UIxMailPartICalActions.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2014 Inverse inc.
+ * Copyright (C) 2007-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  */
 
 #import <Foundation/NSCalendarDate.h>
-#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSDictionary.h>
 #import <Foundation/NSValue.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
@@ -31,10 +31,6 @@
 #import <NGExtensions/NSObject+Logs.h>
 
 #import <NGCards/iCalCalendar.h>
-#import <NGCards/iCalEvent.h>
-#import <NGCards/iCalPerson.h>
-
-#import <UI/Common/WODirectAction+SOGo.h>
 
 #import <NGImap4/NGImap4EnvelopeAddress.h>
 
@@ -44,8 +40,9 @@
 #import <Appointments/SOGoAppointmentFolder.h>
 #import <Appointments/SOGoAppointmentFolders.h>
 #import <Mailer/SOGoMailObject.h>
-#import <SOGo/SOGoParentFolder.h>
 #import <SOGo/SOGoUser.h>
+#import <SOGo/SOGoUserManager.h>
+#import <SOGo/NSString+Utilities.h>
 #import <Mailer/SOGoMailBodyPart.h>
 
 #import "UIxMailPartICalActions.h"
@@ -238,21 +235,20 @@
 - (WOResponse *) _changePartStatusAction: (NSString *) newStatus
                             withDelegate: (iCalPerson *) delegate
 {
-  WOResponse *response;
   SOGoAppointmentObject *eventObject;
+  WOResponse *response;
   iCalEvent *chosenEvent;
-  //NSException *ex;
+  iCalAlarm *alarm;
 
   chosenEvent = [self _setupChosenEventAndEventObject: &eventObject];
   if (chosenEvent)
     {
+      // For invitations, we take the organizers's alarm to start with
+      alarm = [[chosenEvent alarms] lastObject];      
       response = (WOResponse*)[eventObject changeParticipationStatus: newStatus
 							withDelegate: delegate
-                                                               alarm: nil
+                                                               alarm: alarm
 						     forRecurrenceId: [chosenEvent recurrenceId]];
-//      if (ex)
-//	response = ex; //[self responseWithStatus: 500];
-//      else
       if (!response)
 	response = [self responseWith204];
     }
@@ -288,32 +284,36 @@
 //  BOOL receiveUpdates;
   NSString *delegatedEmail, *delegatedUid;
   iCalPerson *delegatedAttendee;
+  NSDictionary *content;
   SOGoUser *user;
   WORequest *request;
   WOResponse *response;
 
   request = [context request];
-  delegatedEmail = [request formValueForKey: @"to"];
+  content = [[request contentAsString] objectFromJSONString];
+  delegatedEmail = [content objectForKey: @"delegatedTo"];
+
   if ([delegatedEmail length])
     {
       user = [context activeUser];
       delegatedAttendee = [iCalPerson new];
       [delegatedAttendee autorelease];
       [delegatedAttendee setEmail: delegatedEmail];
-      delegatedUid = [delegatedAttendee uid];
+      delegatedUid = [delegatedAttendee uidInDomain: [user domain]];
       if (delegatedUid)
 	{
 	  SOGoUser *delegatedUser;
 	  delegatedUser = [SOGoUser userWithLogin: delegatedUid];
 	  [delegatedAttendee setCn: [delegatedUser cn]];
 	}
+
       [delegatedAttendee setRole: @"REQ-PARTICIPANT"];
       [delegatedAttendee setRsvp: @"TRUE"];
       [delegatedAttendee setParticipationStatus: iCalPersonPartStatNeedsAction];
       [delegatedAttendee setDelegatedFrom:
 	       [NSString stringWithFormat: @"mailto:%@", [[user allEmails] objectAtIndex: 0]]];
       
-//      receiveUpdates = [[request formValueForKey: @"receiveUpdates"] boolValue];
+//      receiveUpdates = [[content objectForKey: @"receiveUpdates"] boolValue];
 //      if (receiveUpdates)
 //	[delegatedAttendee setRole: @"NON-PARTICIPANT"];
 
@@ -381,15 +381,50 @@
 
 - (iCalPerson *) _emailParticipantWithEvent: (iCalEvent *) event
 {
-  NSString *emailFrom;
-  SOGoMailObject *mailObject;
   NGImap4EnvelopeAddress *address;
+  SOGoMailObject *mailObject;
+  NSString *emailFrom;
+  iCalPerson *p;
 
   mailObject = [[self clientObject] mailObject];
   address = [[mailObject fromEnvelopeAddresses] objectAtIndex: 0];
   emailFrom = [address baseEMail];
+  p = [event findAttendeeWithEmail: emailFrom];
 
-  return [event findAttendeeWithEmail: emailFrom];
+  // We haven't found it yet, let's look in the identities
+  // associated to this user
+  if (!p)
+    {
+      SOGoUserManager *sm;
+      NSString *uid;
+
+      sm = [SOGoUserManager sharedUserManager];
+      uid = [sm getUIDForEmail: emailFrom];
+
+      if (uid)
+	{
+	  NSArray *allEmails;
+	  NSString *email;
+	  SOGoUser *u;
+	  int i;
+
+	  u = [SOGoUser userWithLogin: uid];
+	  allEmails = [u allEmails];
+	  for (i = 0; i < [allEmails count]; i++)
+	    {
+	      email = [allEmails objectAtIndex: i];
+	      if ([email caseInsensitiveCompare: emailFrom] == NSOrderedSame)
+		continue;
+
+	      p = [event findAttendeeWithEmail: email];
+
+	      if (p)
+		break;
+	    }
+	}
+    }
+
+  return p;
 }
 
 - (BOOL) _updateParticipantStatusInEvent: (iCalEvent *) calendarEvent
