@@ -1,6 +1,6 @@
 /* UIxJSONPreferences.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2017 Inverse inc.
+ * Copyright (C) 2007-2020 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 
 #import <SOPE/NGCards/iCalRecurrenceRule.h>
 
+#import <SOGo/NSArray+Utilities.h>
 #import <SOGo/NSObject+Utilities.h>
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoDomainDefaults.h>
@@ -171,6 +172,9 @@ static SoProduct *preferencesProduct = nil;
   if (![[defaults source] objectForKey: @"SOGoAnimationMode"])
     [[defaults source] setObject: [defaults animationMode]  forKey: @"SOGoAnimationMode"];
 
+  if (![[defaults source] objectForKey: @"SOGoGoogleAuthenticatorEnabled"])
+    [[defaults source] setObject: [NSNumber numberWithBool: NO]  forKey: @"SOGoGoogleAuthenticatorEnabled"];
+
   //
   // Default Calendar preferences
   //
@@ -296,14 +300,16 @@ static SoProduct *preferencesProduct = nil;
   //
   if (![defaults contactsCategories])
     {
-      categoryLabels = [[[self labelForKey: @"contacts_category_labels"  withResourceManager: [preferencesProduct resourceManager]]
-                          componentsSeparatedByString: @","]
-                          sortedArrayUsingSelector: @selector (localizedCaseInsensitiveCompare:)];
+      NSArray *contactsCategories;
 
-      if (!categoryLabels)
-        categoryLabels = [NSArray array];
+      contactsCategories = [[[[self labelForKey: @"contacts_category_labels"  withResourceManager: [preferencesProduct resourceManager]]
+                              componentsSeparatedByString: @","] trimmedComponents]
+                             sortedArrayUsingSelector: @selector (localizedCaseInsensitiveCompare:)];
 
-      [defaults setContactsCategories: categoryLabels];
+      if (!contactsCategories)
+        contactsCategories = [NSArray array];
+
+      [defaults setContactsCategories: contactsCategories];
     }
 
   if (![[defaults source] objectForKey: @"SOGoMailAddOutgoingAddresses"])
@@ -362,6 +368,11 @@ static SoProduct *preferencesProduct = nil;
       [defaults setMailLabelsColors: v];
     }
 
+  if ([domainDefaults forwardEnabled] && ![defaults forwardOptions])
+    {
+      [defaults setForwardOptions: [NSDictionary new]];
+    }
+
   if ([[defaults source] dirty])
     [defaults synchronize];
 
@@ -423,10 +434,13 @@ static SoProduct *preferencesProduct = nil;
     }
   if (account)
     [accounts insertObject: account  atIndex: 0];
+  [values setObject: accounts  forKey: @"AuxiliaryMailAccounts"];
+
+  // Ignore parameters as they are injected in the system mail account ([SOGoUser mailAccounts])
+  [values removeObjectForKey: @"SOGoMailIdentities"];
   [values removeObjectForKey: @"SOGoMailCertificate"];
   [values removeObjectForKey: @"SOGoMailCertificateAlwaysSign"];
   [values removeObjectForKey: @"SOGoMailCertificateAlwaysEncrypt"];
-  [values setObject: accounts  forKey: @"AuxiliaryMailAccounts"];
 
   // Add the domain's default vacation subject if user has not specified a custom subject
   vacationOptions = [defaults vacationOptions];
