@@ -1,6 +1,6 @@
 /* UIxPreferences.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2019 Inverse inc.
+ * Copyright (C) 2007-2020 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -113,10 +113,6 @@ static NSArray *reminderValues = nil;
         referenceDate = [referenceDate addYear:0 month:0 day:(-[referenceDate dayOfMonth] + 1) hour:0 minute:0 second:0];
       ASSIGN (today, referenceDate);
 
-      calendarCategories = nil;
-      calendarCategoriesColors = nil;
-      category = nil;
-
       ASSIGN (daysOfWeek, [locale objectForKey: NSWeekDayNameArray]);
 
       dd = [user domainDefaults];
@@ -157,10 +153,6 @@ static NSArray *reminderValues = nil;
   [user release];
   [sieveFilters release];
   [vacationOptions release];
-  [calendarCategories release];
-  [calendarCategoriesColors release];
-  [category release];
-  [contactsCategories release];
   [forwardOptions release];
   [daysOfWeek release];
   [addressBooksIDWithDisplayName release];
@@ -793,23 +785,9 @@ static NSArray *reminderValues = nil;
 //
 // Used internally
 //
-- (NSString *) _defaultEmailAddresses
+- (NSString *) defaultEmailAddresses
 {
-  NSArray *addressesList;
-  NSMutableArray *uniqueAddressesList;
-  NSString *address;
-  unsigned int i;
-
-  uniqueAddressesList = [NSMutableArray array];
-  addressesList = [NSMutableArray arrayWithArray: [user allEmails]];
-  for (i = 0; i < [addressesList count]; i++)
-    {
-      address = [addressesList objectAtIndex: i];
-      if (![uniqueAddressesList containsObject: address])
-	[uniqueAddressesList addObject: address];
-    }
-
-  return [uniqueAddressesList componentsJoinedByString: @", "];
+  return [[[user allEmails] uniqueObjects] jsonRepresentation];
 }
 
 //
@@ -880,15 +858,18 @@ static NSArray *reminderValues = nil;
 //
 // Used by templates
 //
-- (NSString *) autoReplyEmailAddresses
+- (NSString *) defaultContactsCategories
 {
-  NSArray *addressesList;
+  NSArray *contactsCategories;
 
-  addressesList = [vacationOptions objectForKey: @"autoReplyEmailAddresses"];
+  contactsCategories = [[[[self labelForKey: @"contacts_category_labels"  withResourceManager: [self resourceManager]]
+                              componentsSeparatedByString: @","] trimmedComponents]
+                             sortedArrayUsingSelector: @selector (localizedCaseInsensitiveCompare:)];
 
-  return (addressesList
-          ? [addressesList componentsJoinedByString: @", "]
-          : [self _defaultEmailAddresses]);
+  if (!contactsCategories)
+    contactsCategories = [NSArray array];
+
+  return [contactsCategories jsonRepresentation];
 }
 
 //
@@ -1021,6 +1002,20 @@ static NSArray *reminderValues = nil;
   return [NSString stringWithString: SOGoVersion];
 }
 
+- (BOOL) isGoogleAuthenticatorEnabled
+{
+#if defined(MFA_CONFIG)
+  return YES;
+#else
+  return NO;
+#endif
+}
+
+- (NSString *) googleAuthenticatorKey
+{
+  return [[context activeUser] googleAuthenticatorKey];
+}
+
 //
 // Used internally
 //
@@ -1124,63 +1119,6 @@ static NSArray *reminderValues = nil;
   return [[user domainDefaults] mailAuxiliaryUserAccountsEnabled];
 }
 
-//
-// Used internally
-//
-- (void) _extractMainIdentity: (NSDictionary *) identity
-                 inDictionary: (NSMutableDictionary *) target
-
-{
-  /* We perform some validation here as we have no guaranty on the input
-     validity. */
-  NSString *value;
-
-  if ([identity isKindOfClass: [NSDictionary class]])
-    {
-      value = [identity objectForKey: @"signature"];
-
-      if (value)
-        [target setObject: value  forKey: @"SOGoMailSignature"];
-      else
-        [target removeObjectForKey: @"SOGoMailSignature"];
-
-      if (mailCustomFromEnabled)
-        {
-          value = [[identity objectForKey: @"email"]
-                    stringByTrimmingSpaces];
-
-          /* We make sure that the "custom" value is different from the system email */
-          if ([value length] == 0
-              || [[user systemEmail] isEqualToString: value])
-            value = nil;
-
-          if (value)
-            [target setObject: value  forKey: @"SOGoMailCustomEmail"];
-          else
-            [target removeObjectForKey: @"SOGoMailCustomEmail"];
-
-          value = [[identity objectForKey: @"fullName"]
-                    stringByTrimmingSpaces];
-          if ([value length] == 0
-              || [[user cn] isEqualToString: value])
-            value = nil;
-
-          if (value)
-            [target setObject: value  forKey: @"SOGoMailCustomFullName"];
-          else
-            [target removeObjectForKey: @"SOGoMailCustomFullName"];
-        }
-
-      value = [[identity objectForKey: @"replyTo"]
-                stringByTrimmingSpaces];
-
-      if (value && [value length] > 0)
-        [target setObject: value  forKey: @"SOGoMailReplyTo"];
-      else
-        [target removeObjectForKey: @"SOGoMailReplyTo"];
-    }
-}
-
 //
 // Used internally
 //
@@ -1243,20 +1181,6 @@ static NSArray *reminderValues = nil;
     }
 }
 
-//
-// Used internally
-//
-- (void) _extractMainCustomFrom: (NSDictionary *) account
-{
-}
-
-//
-// Used internally
-//
-- (void) _extractMainReplyTo: (NSDictionary *) account
-{
-}
-
 //
 // Used internally
 //
@@ -1273,7 +1197,7 @@ static NSArray *reminderValues = nil;
   if (!knownKeys)
     {
       knownKeys = [NSArray arrayWithObjects: @"fullName", @"email",
-                           @"signature", @"replyTo", nil];
+                           @"signature", @"replyTo", @"isDefault", nil];
       [knownKeys retain];
     }
 
@@ -1327,7 +1251,7 @@ static NSArray *reminderValues = nil;
 
   if (!knownKeys)
     {
-      knownKeys = [NSArray arrayWithObjects: @"name", @"serverName", @"port",
+      knownKeys = [NSArray arrayWithObjects: @"id", @"name", @"serverName", @"port",
                            @"userName", @"password", @"encryption", @"replyTo",
                            @"identities", @"mailboxes",
                            @"receipts", @"security", @"isNew",
@@ -1381,9 +1305,8 @@ static NSArray *reminderValues = nil;
   if ([account isKindOfClass: [NSDictionary class]])
     {
       identities = [account objectForKey: @"identities"];
-      if ([identities isKindOfClass: [NSArray class]]
-          && [identities count] > 0)
-        [self _extractMainIdentity: [identities objectAtIndex: 0]  inDictionary: target];
+      if ([self _validateAccountIdentities: identities])
+        [target setObject: identities forKey: @"SOGoMailIdentities"];
       [self _extractMainReceiptsPreferences: [account objectForKey: @"receipts"]  inDictionary: target];
       [self _extractMainSecurityPreferences: [account objectForKey: @"security"]  inDictionary: target];
     }
@@ -1581,7 +1504,7 @@ static NSArray *reminderValues = nil;
         [v setObject: [[user userDefaults] mailCertificate]  forKey: @"SOGoMailCertificate"];
 
       //
-      // We sanitize our auxilary mail accounts
+      // We sanitize our auxiliary mail accounts
       //
       accounts = [v objectForKey: @"AuxiliaryMailAccounts"];
       if (accounts && [accounts isKindOfClass: [NSArray class]])
@@ -1589,10 +1512,6 @@ static NSArray *reminderValues = nil;
           if ([accounts count] > 0)
             {
               // The first account is the main system account. The following mapping is required:
-              // - identities[0].signature             => SOGoMailSignature
-              // - identities[0].email                 => SOGoMailCustomEmail
-              // - identities[0].fullName              => SOGoMailCustomFullName
-              // - identities[0].replyTo               => SOGoMailReplyTo
               // - receipts.receiptAction              => SOGoMailReceiptAllow
               // - receipts.receiptNonRecipientAction  => SOGoMailReceiptNonRecipientAction
               // - receipts.receiptOutsideDomainAction => SOGoMailReceiptOutsideDomainAction
