@@ -43,6 +43,7 @@
 
 
     _registerHotkeys(hotkeys);
+    _loadCertificate();
 
     $scope.$on('$destroy', function() {
       // Deregister hotkeys
@@ -58,7 +59,7 @@
           key: hotkey,
           description: l('Delete'),
           callback: function($event) {
-            if (vm.currentFolder.$selectedCount() === 0)
+            if (vm.currentFolder.acls.objectEraser && vm.currentFolder.$selectedCount() === 0)
               confirmDelete();
             $event.preventDefault();
           }
@@ -71,6 +72,15 @@
       });
     }
 
+    function _loadCertificate() {
+      if (vm.card.hasCertificate)
+        vm.card.$certificate().then(function(crt) {
+          vm.certificate = crt;
+        }, function() {
+          delete vm.card.hasCertificate;
+        });
+    }
+
     function transformCategory(input) {
       if (angular.isString(input))
         return { value: input };
@@ -100,7 +110,7 @@
       focus('phone_' + i);
     }
     function addUrl() {
-      var i = vm.card.$addUrl('', '');
+      var i = vm.card.$addUrl('', 'https://www.fsf.org/');
       focus('url_' + i);
     }
     function canAddCustomField() {
