@@ -104,9 +104,12 @@
     var collection = [],
         // Local recursive function
         createMailboxes = function(level, mailbox) {
+          mailbox.isSentFolder = mailbox.isSentFolder || mailbox.type == 'sent';
           for (var i = 0; i < mailbox.children.length; i++) {
             mailbox.children[i].level = level;
             mailbox.children[i] = new Mailbox(account, mailbox.children[i]);
+            if (mailbox.isSentFolder)
+              mailbox.children[i].isSentFolder = true;
             createMailboxes(level+1, mailbox.children[i]);
           }
         };
@@ -203,7 +206,7 @@
       }
       else {
         this.$isSpecial = false;
-        this.$icon = 'folder_open';
+        this.$icon = 'folder';
       }
     }
     this.$isNoInferiors = this.isNoInferiors();
@@ -611,7 +614,7 @@
    * @returns true if folder is eligible
    */
   Mailbox.prototype.$canFolderAs = function() {
-    return this.type == 'folder' && this.level === 0;
+    return this.type == 'folder';
   };
 
   /**
@@ -786,13 +789,14 @@
       var currentUids = uids.slice(start, end),
           currentMessages = messages.slice(start, end),
           data = { uids: currentUids };
+      if (options) angular.extend(data, options);
       return Mailbox.$$resource.post(_this.id, 'batchDelete', data).then(function(data) {
         if (end < uids.length) {
           _this.$_deleteMessages(currentUids, currentMessages);
           return _deleteMessages(end, Math.min(end + batchSize, uids.length));
         }
         else {
-          // Update inbox quota
+          // Last API call; update inbox quota
           if (data.quotas)
             _this.$account.updateQuota(data.quotas);
           return _this.$_deleteMessages(currentUids, currentMessages);
