@@ -53,7 +53,11 @@
           key: hotkey,
           description: l('Delete selected message or folder'),
           callback: function() {
-            if (Mailbox.selectedFolderController && Mailbox.selectedFolder && Mailbox.selectedFolder.$isEditable && !Mailbox.selectedFolder.hasSelectedMessage())
+            if (Mailbox.selectedFolderController &&
+                Mailbox.selectedFolder &&
+                Mailbox.selectedFolder.$isEditable &&
+                !Mailbox.selectedFolder.hasSelectedMessage() &&
+                Mailbox.selectedFolder.$selectedCount() === 0)
               Mailbox.selectedFolderController.confirmDelete(Mailbox.selectedFolder);
           }
         }));
@@ -297,8 +301,9 @@
       Account.$$resource.post('', 'unseenCount', {mailboxes: unseenCountFolders}).then(function(data) {
         _.forEach(vm.accounts, function(account) {
           _.forEach(account.$$flattenMailboxes, function(mailbox) {
-            if (data[mailbox.id])
+            if (data[mailbox.id]) {
               mailbox.unseenCount = data[mailbox.id];
+            }
           });
         });
       });
@@ -336,7 +341,7 @@
           $state.go('mail.account.mailbox');
         $mdToast.show(
           $mdToast.simple()
-            .content(success)
+            .textContent(success)
             .position('top right')
             .hideDelay(2000));
       });
