@@ -32,7 +32,7 @@
         '  </div>',
         '</div>',
         '<div class="sg-tile-icons">',
-        '  <md-icon class="ng-hide">star</md-icon>',
+        '  <md-icon class="ng-hide sg-icon-star">star</md-icon>',
         '  <md-icon class="ng-hide">reply</md-icon>',
         '  <md-icon class="ng-hide">forward</md-icon>',
         '  <md-icon class="ng-hide">attach_file</md-icon>',
@@ -140,7 +140,7 @@
           $ctrl.mailboxNameElement.innerHTML = $ctrl.message.$mailbox.$displayName;
 
         // Sender or recipient when in
-        if ($ctrl.MailboxService.selectedFolder.type == 'sent')
+        if ($ctrl.MailboxService.selectedFolder.isSentFolder)
           $ctrl.senderElement.innerHTML = $ctrl.message.$shortAddress('to').encodeEntities();
         else
           $ctrl.senderElement.innerHTML = $ctrl.message.$shortAddress('from').encodeEntities();
