@@ -7,22 +7,30 @@
   /**
    * @ngInject
    */
-  PreferencesController.$inject = ['$q', '$window', '$state', '$mdMedia', '$mdSidenav', '$mdDialog', '$mdToast', 'sgSettings', 'sgFocus', 'Dialog', 'User', 'Account', 'Preferences', 'Authentication'];
-  function PreferencesController($q, $window, $state, $mdMedia, $mdSidenav, $mdDialog, $mdToast, sgSettings, focus, Dialog, User, Account, Preferences, Authentication) {
+  PreferencesController.$inject = ['$q', '$window', '$state', '$mdConstant', '$mdMedia', '$mdSidenav', '$mdDialog', '$mdToast', 'sgSettings', 'sgFocus', 'Dialog', 'User', 'Account', 'Preferences', 'Authentication'];
+  function PreferencesController($q, $window, $state, $mdConstant, $mdMedia, $mdSidenav, $mdDialog, $mdToast, sgSettings, focus, Dialog, User, Account, Preferences, Authentication) {
     var vm = this, mailboxes = [], today = new Date().beginOfDay();
 
     this.$onInit = function() {
       this.preferences = Preferences;
-      this.passwords = { newPassword: null, newPasswordConfirmation: null };
+      this.passwords = { newPassword: null, newPasswordConfirmation: null, oldPassword: null };
       this.timeZonesList = $window.timeZonesList;
       this.timeZonesSearchText = '';
       this.sieveVariablesCapability = ($window.sieveCapabilities.indexOf('variables') >= 0);
+      this.addressesSearchText = '';
       this.mailLabelKeyRE = new RegExp(/^(?!^_\$)[^(){} %*\"\\\\]*?$/);
+      this.emailSeparatorKeys = [
+        $mdConstant.KEY_CODE.ENTER,
+        $mdConstant.KEY_CODE.TAB,
+        $mdConstant.KEY_CODE.COMMA,
+        $mdConstant.KEY_CODE.SEMICOLON
+      ];
 
       // Set alternate avatar in User service
       if (Preferences.defaults.SOGoAlternateAvatar)
         User.$alternateAvatar = Preferences.defaults.SOGoAlternateAvatar;
 
+      this.preferences.hasActiveExternalSieveScripts();
       this.updateVacationDates();
     };
 
@@ -47,6 +55,16 @@
         });
     };
 
+    this.onDesktopNotificationsChange = function() {
+      if (this.preferences.defaults.SOGoDesktopNotifications)
+        this.preferences.authorizeNotifications();
+    };
+
+    this.resetContactsCategories = function(form) {
+      this.preferences.defaults.SOGoContactsCategories = $window.defaultContactsCategories;
+      form.$setDirty();
+    };
+
     this.resetCalendarCategories = function(form) {
       this.preferences.defaults.SOGoCalendarCategories = _.keys($window.defaultCalendarCategories);
       this.preferences.defaults.SOGoCalendarCategoriesColorsValues = _.values($window.defaultCalendarCategories);
@@ -90,28 +108,26 @@
     };
 
     this.addMailAccount = function(ev, form) {
-      var account;
+      var account, index;
 
-      this.preferences.defaults.AuxiliaryMailAccounts.push({});
-
-      account = _.last(this.preferences.defaults.AuxiliaryMailAccounts);
-      angular.extend(account,
-                     {
-                       isNew: true,
-                       name: "",
-                       identities: [
-                         {
-                           fullName: "",
-                           email: ""
-                         }
-                       ],
-                       receipts: {
-                         receiptAction: "ignore",
-                         receiptNonRecipientAction: "ignore",
-                         receiptOutsideDomainAction: "ignore",
-                         receiptAnyAction: "ignore"
-                       }
-                     });
+      index = this.preferences.defaults.AuxiliaryMailAccounts.length;
+      account = new Account({
+        id: index,
+        isNew: true,
+        name: "",
+        identities: [
+          {
+            fullName: "",
+            email: ""
+          }
+        ],
+        receipts: {
+          receiptAction: "ignore",
+          receiptNonRecipientAction: "ignore",
+          receiptOutsideDomainAction: "ignore",
+          receiptAnyAction: "ignore"
+        }
+      });
 
       $mdDialog.show({
         controller: 'AccountDialogController',
@@ -121,18 +137,26 @@
         locals: {
           defaults: this.preferences.defaults,
           account: account,
-          accountId: (this.preferences.defaults.AuxiliaryMailAccounts.length-1),
+          accountId: index,
           mailCustomFromEnabled: $window.mailCustomFromEnabled
         }
       }).then(function() {
+        // Automatically expand the new mail account
+        if (!angular.isArray(vm.preferences.settings.Mail.ExpandedFolders)) {
+          vm.preferences.settings.Mail.ExpandedFolders = ['/0'];
+        }
+        vm.preferences.settings.Mail.ExpandedFolders.push('/' + index);
+        vm.preferences.defaults.AuxiliaryMailAccounts.push(account.$omit());
+
         form.$setDirty();
-      }).catch(function() {
-        vm.preferences.defaults.AuxiliaryMailAccounts.pop();
       });
     };
 
     this.editMailAccount = function(event, index, form) {
-      var account = this.preferences.defaults.AuxiliaryMailAccounts[index];
+      var data, account;
+
+      data = _.assign({ id: index }, _.cloneDeep(this.preferences.defaults.AuxiliaryMailAccounts[index]));
+      account = new Account(data);
       $mdDialog.show({
         controller: 'AccountDialogController',
         controllerAs: '$AccountDialogController',
@@ -145,11 +169,9 @@
           mailCustomFromEnabled: $window.mailCustomFromEnabled
         }
       }).then(function() {
-        vm.preferences.defaults.AuxiliaryMailAccounts[index] = account;
+        vm.preferences.defaults.AuxiliaryMailAccounts[index] = account.$omit();
         form.$setDirty();
-      }, function() {
-        // Cancel
-      });
+      }).catch(_.noop); // Cancel
     };
 
     this.removeMailAccount = function(index, form) {
@@ -209,7 +231,8 @@
         locals: {
           filter: filter,
           mailboxes: mailboxes,
-          labels: this.preferences.defaults.SOGoMailLabelsColors
+          labels: this.preferences.defaults.SOGoMailLabelsColors,
+          validateForwardAddress: validateForwardAddress
         }
       }).then(function() {
         if (!vm.preferences.defaults.SOGoSieveFilters)
@@ -231,7 +254,8 @@
         locals: {
           filter: filter,
           mailboxes: mailboxes,
-          labels: this.preferences.defaults.SOGoMailLabelsColors
+          labels: this.preferences.defaults.SOGoMailLabelsColors,
+          validateForwardAddress: validateForwardAddress
         }
       }).then(function() {
         vm.preferences.defaults.SOGoSieveFilters[index] = filter;
@@ -255,19 +279,29 @@
       return this._onFiltersOrderChanged;
     };
 
+    this.filterEmailAddresses = function ($query) {
+      return _.filter(
+        _.difference($window.defaultEmailAddresses,
+                     this.preferences.defaults.Vacation.autoReplyEmailAddresses),
+        function (address) {
+          return address.toLowerCase().indexOf($query.toLowerCase()) >= 0;
+        }
+      );
+    };
+
     this.addDefaultEmailAddresses = function(form) {
       var v = [];
 
       if (angular.isDefined(this.preferences.defaults.Vacation.autoReplyEmailAddresses)) {
-        v = this.preferences.defaults.Vacation.autoReplyEmailAddresses.split(',');
+        v = this.preferences.defaults.Vacation.autoReplyEmailAddresses;
       }
 
-      this.preferences.defaults.Vacation.autoReplyEmailAddresses = (_.union($window.defaultEmailAddresses.split(','), v)).join(',');
+      this.preferences.defaults.Vacation.autoReplyEmailAddresses = _.union($window.defaultEmailAddresses, v);
       form.$setDirty();
     };
 
     this.userFilter = function(search, excludedUsers) {
-      if (search.length < sgSettings.minimumSearchLength())
+      if (!search || search.length < sgSettings.minimumSearchLength())
         return [];
 
       return User.$filter(search, excludedUsers).then(function(users) {
@@ -285,7 +319,7 @@
     };
 
     this.manageSieveScript = function(form) {
-      this.preferences.defaults.hasActiveExternalSieveScripts = false;
+      this.preferences.hasActiveExternalSieveScripts(false);
       form.$setDirty();
     };
 
@@ -317,23 +351,18 @@
       }
     };
 
-    this.save = function(form, options) {
-      var i, sendForm, addresses, defaultAddresses, domains, domain;
+    function validateForwardAddress(address) {
+      var defaultAddresses, domains, domain;
 
-      sendForm = true;
       domains = [];
 
-      // We do some sanity checks
       if ($window.forwardConstraints > 0 &&
-          angular.isDefined(this.preferences.defaults.Forward) &&
-          this.preferences.defaults.Forward.enabled &&
-          angular.isDefined(this.preferences.defaults.Forward.forwardAddress)) {
-
-        addresses = this.preferences.defaults.Forward.forwardAddress.split(",");
+          angular.isDefined(Preferences.defaults.Forward) &&
+          Preferences.defaults.Forward.enabled &&
+          angular.isDefined(Preferences.defaults.Forward.forwardAddress)) {
 
         // We first extract the list of 'known domains' to SOGo
-        defaultAddresses = $window.defaultEmailAddresses.split(/, */);
-
+        defaultAddresses = $window.defaultEmailAddresses;
         _.forEach(defaultAddresses, function(adr) {
           var domain = adr.split("@")[1];
           if (domain) {
@@ -342,22 +371,40 @@
         });
 
         // We check if we're allowed or not to forward based on the domain defaults
-        for (i = 0; i < addresses.length && sendForm; i++) {
-          domain = addresses[i].split("@")[1].toLowerCase();
-          if (domains.indexOf(domain) < 0 && $window.forwardConstraints == 1) {
-            Dialog.alert(l('Error'), l("You are not allowed to forward your messages to an external email address."));
-            sendForm = false;
-          }
-          else if (domains.indexOf(domain) >= 0 && $window.forwardConstraints == 2) {
-            Dialog.alert(l('Error'), l("You are not allowed to forward your messages to an internal email address."));
-            sendForm = false;
-          }
-          else if ($window.forwardConstraints == 2 &&
-                   $window.forwardConstraintsDomains.length > 0 &&
-                   $window.forwardConstraintsDomains.indexOf(domain) < 0) {
-            Dialog.alert(l('Error'), l("You are not allowed to forward your messages to this domain:") + " " + domain);
-            sendForm = false;
+        domain = address.split("@")[1].toLowerCase();
+        if (domains.indexOf(domain) < 0 && $window.forwardConstraints == 1) {
+          throw new Error(l("You are not allowed to forward your messages to an external email address."));
+        }
+        else if (domains.indexOf(domain) >= 0 && $window.forwardConstraints == 2) {
+          throw new Error(l("You are not allowed to forward your messages to an internal email address."));
+        }
+        else if ($window.forwardConstraints == 2 &&
+                 $window.forwardConstraintsDomains.length > 0 &&
+                 $window.forwardConstraintsDomains.indexOf(domain) < 0) {
+          throw new Error(l("You are not allowed to forward your messages to this domain:") + " " + domain);
+        }
+      }
+
+      return true;
+    }
+
+    this.save = function(form, options) {
+      var i, sendForm, addresses;
+
+      sendForm = true;
+
+      // We do some sanity checks
+
+      // We check if we're allowed or not to forward based on the domain defaults
+      if (this.preferences.defaults.Forward && this.preferences.defaults.Forward.forwardAddress) {
+        addresses = this.preferences.defaults.Forward.forwardAddress;
+        try {
+          for (i = 0; i < addresses.length; i++) {
+            validateForwardAddress(addresses[i]);
           }
+        } catch (err) {
+          Dialog.alert(l('Error'), err);
+          sendForm = false;
         }
       }
 
@@ -410,30 +457,39 @@
           if (!options || !options.quick) {
             $mdToast.show(
               $mdToast.simple()
-                .content(l('Preferences saved'))
+                .textContent(l('Preferences saved'))
                 .position('bottom right')
                 .hideDelay(2000));
             form.$setPristine();
           }
         });
 
-      return $q.reject();
+      return $q.reject('Invalid form');
     };
 
-    this.canChangePassword = function() {
+    this.canChangePassword = function(form) {
+      if (this.passwords.newPasswordConfirmation && this.passwords.newPasswordConfirmation.length &&
+          this.passwords.newPassword != this.passwords.newPasswordConfirmation) {
+        form.newPasswordConfirmation.$setValidity('newPasswordMismatch', false);
+        return false;
+      }
+      else {
+        form.newPasswordConfirmation.$setValidity('newPasswordMismatch', true);
+      }
       if (this.passwords.newPassword && this.passwords.newPassword.length > 0 &&
           this.passwords.newPasswordConfirmation && this.passwords.newPasswordConfirmation.length &&
-          this.passwords.newPassword == this.passwords.newPasswordConfirmation)
+          this.passwords.newPassword == this.passwords.newPasswordConfirmation &&
+          this.passwords.oldPassword && this.passwords.oldPassword.length > 0)
         return true;
 
       return false;
     };
 
     this.changePassword = function() {
-      Authentication.changePassword(this.passwords.newPassword).then(function() {
+      Authentication.changePassword(this.passwords.newPassword, this.passwords.oldPassword).then(function() {
         var alert = $mdDialog.alert({
           title: l('Password'),
-          content: l('The password was changed successfully.'),
+          textContent: l('The password was changed successfully.'),
           ok: l('OK')
         });
         $mdDialog.show( alert )
